; @author Sylwester Arabas <slayoo@igf.fuw.edu.pl>

pro ffssp_ncdf
  ; compilation options
  compile_opt idl2, strictarr;

  ; config
  version = '0.1';
  datadir = "/home/pace/resu/RICO";
  flights = ["rf06", "rf07", "rf08", "rf09", "rf11", "rf12", "rf13"];
;  flights = ["rf06"];
  fastfssprange = [3., 50.]; [micrometers]

  ; constants (dim indices)
  dim_Time_idx = 0L;
  dim_sps10_idx = 1L;
  dim_Vector19_idx = 2L;

  ; constants (var indices)
  base_time_idx = 0L;
  Time_idx = 1L;
  time_offset_idx = 2L;
  FSTFSSP_CONC_idx = 3L;
  FSTFSSP_CTOT_idx = 4L;
  FSTFSSP_LWC_idx = 5L;
  FSTFSSP_DINT_idx = 6L;
  FSTFSSP_DAVG_idx = 7L;
  FSTFSSP_REFF_idx = 8L;
  FSTFSSP_DVOL_idx = 9L;

  indices_0d = [base_time_idx];
  indices_long = [Time_idx, time_offset_idx];
  indices_float = [FSTFSSP_CTOT_idx, FSTFSSP_LWC_idx, FSTFSSP_DINT_idx, FSTFSSP_DAVG_idx, FSTFSSP_REFF_idx, FSTFSSP_DVOL_idx, FSTFSSP_CONC_idx];

  ; memalloc
  dims = intarr(3, /nozero);
  vars = intarr(10, /nozero);
  line = '';
  tmpdata1 = dblarr(45, /nozero);
  tmpdata10 = dblarr(45, 10, /nozero);
  ncov = ptrarr(n_elements(vars), /nozero);
  mapl = lonarr(n_elements(vars), /nozero);
  mapr = lonarr(n_elements(vars), /nozero);

  ; constants (text-file map)
  offs = replicate(0D, n_elements(vars));
  mplr = replicate(1D, n_elements(vars));
  ;   FSTFSSP_CONC
  mapl[FSTFSSP_CONC_idx] = 26L;
  mapr[FSTFSSP_CONC_idx] = 44L;
  mplr[FSTFSSP_CONC_idx] = .001;
  ;   FSTFSSP_CTOT
  mapl[FSTFSSP_CTOT_idx] = 17L & mapr[FSTFSSP_CTOT_idx] = 17L;
  ;   FSTFSSP_LWC
  mapl[FSTFSSP_LWC_idx] = 19L & mapr[FSTFSSP_LWC_idx] = 19L;
  mplr[FSTFSSP_LWC_idx] = .001;
  ;   FSTFSSP_DINT
  mapl[FSTFSSP_DINT_idx] = 18L & mapr[FSTFSSP_DINT_idx] = 18L;
  mplr[FSTFSSP_DINT_idx] = .001;
  ;   FSTFSSP_DAVG
  mapl[FSTFSSP_DAVG_idx] = 21L & mapr[FSTFSSP_DAVG_idx] = 21L;
  mplr[FSTFSSP_DAVG_idx] = .001;
  ;   FSTFSSP_REFF
  mapl[FSTFSSP_REFF_idx] = 23L & mapr[FSTFSSP_REFF_idx] = 23L;
  mplr[FSTFSSP_REFF_idx] = .0005;
  ;   FSTFSSP_DVOL
  mapl[FSTFSSP_DVOL_idx] = 22L & mapr[FSTFSSP_DVOL_idx] = 22L;
  mplr[FSTFSSP_DVOL_idx] = .001;
  ;   time-related
  mapl[base_time_idx] = 0L & mapr[base_time_idx] = 0L;
  mplr[base_time_idx] = 0.;
  mapl[Time_idx] = 0L & mapr[Time_idx] = 0L;
  mplr[Time_idx] = .001;
  mapl[time_offset_idx] = 0L & mapr[time_offset_idx] = 0L;
  mplr[time_offset_idx] = .001;

  ; other useful things
  datearr = bin_date(systime());
  spawn, "date +'%Y-%m-%d %H:%M:%S %z'", datestring
  datestring = datestring[0];

  ; loop over flights
  for flight = 0L, n_elements(flights) - 1 do begin

    numrec = 0L;

    ; geting some compatibility-info from the NCER RAF netCDF file
    filename = file_search(datadir + "/" + flights[flight] + "/" + "*.HRT.*.nc");
    filename = filename[0];
    print, flights[flight] + ": opening " + filename
    ncdf = ncdf_open(filename, /nowrite);

    i = ncdf_varid(ncdf, "base_time");
    ncdf_varget, ncdf, 0, i;
    offs[base_time_idx] = i;

    i = ncdf_dimid(ncdf, "Time");
    ncdf_diminq, ncdf, i, null, Timesize;

    spawn, "date -u -d @" + strtrim(string(i), 2) + " +'%Y-%m-%d %H:%M:%S %z'", base_time_string
    base_time_string = base_time_string[0];

    ncdf_attget, ncdf, "TimeInterval", timeinterval, /global;
    ncdf_attget, ncdf, "FlightDate", flightdate, /global;

    ncdf_close, ncdf;

    ; opening the text file
    filename = file_search(datadir + "/" + flights[flight] + "/" + "*R0010");
    filename = filename[0];
    get_lun, unit
    print, flights[flight] + ": opening " + filename
    openr, unit, filename

    ; text file first-pass
    print, flights[flight] + ": 1st pass on " + filename
    while eof(unit) eq 0 do begin
      ; reading line data
      readf, unit, format='(%"%s")', line;
      ; skipping comments and incrementing record counter
      if (strmid(line, 0, 1) ne '#') then numrec = numrec + 1L;
    endwhile

    ; time offset negotiation and time-dimension length check
    reads, strmid(file_basename(filename), 12, 2), hh_fast;
    reads, strmid(systime(0, offs[base_time_idx], /utc), 11, 2), hh_raf;
    reads, strmid(file_basename(filename), 14, 2), mm_fast
    reads, strmid(systime(0, offs[base_time_idx], /utc), 14, 2), mm_raf;
    reads, strmid(file_basename(filename), 16, 2), ss_fast;
    reads, strmid(systime(0, offs[base_time_idx], /utc), 17, 2), ss_raf;
    secdiff = long(60L*60L*(hh_fast - hh_raf) + 60L*(mm_fast - mm_raf) + (ss_fast - ss_raf));
    for i=0, n_elements(vars) - 1 do ncov[i] = ptr_new(long([secdiff, 0L]));

    ; -100 miliseconds offset due to difference in counting logic between Meteo-France and RAF
    offs[Time_idx] = secdiff - .1D;
    offs[time_offset_idx] = offs[Time_idx];

    ; creating the file
    print, flights[flight] + ": creating " + filename + ".nc"
    ncdf = ncdf_create(filename + ".nc", /clobber);
    ncdf_attput, ncdf, "Source", "data: Meteo-France; netCDF: IGFUW (slayoo@igf.fuw.edu.pl)", /global
    ncdf_attput, ncdf, "Conventions", "NCAR-RAF/nimbus", /global
    ncdf_attput, ncdf, "DateProcessed", datestring, /global
    ncdf_attput, ncdf, "Version", version, /global
    ncdf_attput, ncdf, "ProjectName", "RICO", /global
    ncdf_attput, ncdf, "Aircraft", "N130AR", /global
    ncdf_attput, ncdf, "ProjectNumber", '135', /global
    ncdf_attput, ncdf, "FlightNumber", flights[flight], /global
    ncdf_attput, ncdf, "FlightDate", string(flightdate), /global
    ncdf_attput, ncdf, "coordinates", "Time", /global
    ncdf_attput, ncdf, "TimeInterval", string(timeinterval), /global
    ncdf_attput, ncdf, "Categories", "PMS Probe,Liquid Water", /global

    ; dimensions definition
    dims[dim_Time_idx] = ncdf_dimdef(ncdf, "Time", Timesize);
    dims[dim_sps10_idx] = ncdf_dimdef(ncdf, "sps10", 10); 
    dims[dim_Vector19_idx] = ncdf_dimdef(ncdf, "Vector19", 19); 

    ; variables definition
    vars[base_time_idx] = ncdf_vardef(ncdf, "base_time", /long);
    ncdf_attput, ncdf, vars[base_time_idx], "units", "seconds since 1970-01-01 00:00:00 +0000";
    ncdf_attput, ncdf, vars[base_time_idx], "long_name", "Start time of data recording.";

    vars[Time_idx] = ncdf_vardef(ncdf, "Time", [dims[dim_Time_idx]], /long);
    ncdf_attput, ncdf, vars[Time_idx], "units", "seconds since " + base_time_string;
    ncdf_attput, ncdf, vars[Time_idx], "long_name", "time of measurement";
    ncdf_attput, ncdf, vars[Time_idx], "standard_name", "time";
    ncov[Time_idx] = ptr_new(long([secdiff]));

    vars[time_offset_idx] = ncdf_vardef(ncdf, "time_offset", [dims[dim_Time_idx]], /float);
    ncdf_attput, ncdf, vars[time_offset_idx], "units", "seconds since " + base_time_string;
    ncdf_attput, ncdf, vars[time_offset_idx], "long_name", "seconds since base_time";
    ncov[time_offset_idx] = ptr_new(long([secdiff]));

    vars[FSTFSSP_CONC_idx] = ncdf_vardef(ncdf, "FSTFSSP_CONC_PERC", [dims[dim_Time_idx], dims[dim_sps10_idx], dims[dim_Vector19_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_CONC_idx], "units", "um";
    ncdf_attput, ncdf, vars[FSTFSSP_CONC_idx], "long_name", "Fast-FSSP Concentration (5th, 10th, ... 95th percentiles)";
    ncdf_attput, ncdf, vars[FSTFSSP_CONC_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_CONC_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_CONC_idx], "CellSizeRange", fastfssprange;
    ncov[FSTFSSP_CONC_idx] = ptr_new(long([secdiff, 0L, 0L]));

    vars[FSTFSSP_CTOT_idx] = ncdf_vardef(ncdf, "FSTFSSP_CTOT", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_CTOT_idx], "units", "#/cm3";
    ncdf_attput, ncdf, vars[FSTFSSP_CTOT_idx], "long_name", "Fast-FSSP Concentration (all cells)";
    ncdf_attput, ncdf, vars[FSTFSSP_CTOT_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_CTOT_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_CTOT_idx], "CellSizeRange", fastfssprange;

    vars[FSTFSSP_LWC_idx] = ncdf_vardef(ncdf, "FSTFSSP_LWC", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_LWC_idx], "units", "gram/m3";
    ncdf_attput, ncdf, vars[FSTFSSP_LWC_idx], "long_name", "Fast-FSSP Liquid Water Content";
    ncdf_attput, ncdf, vars[FSTFSSP_LWC_idx], "Category", "Liquid Water";
    ncdf_attput, ncdf, vars[FSTFSSP_LWC_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_LWC_idx], "CellSizeRange", fastfssprange;

    vars[FSTFSSP_DINT_idx] = ncdf_vardef(ncdf, "FSTFSSP_DINT", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_DINT_idx], "units", "um";
    ncdf_attput, ncdf, vars[FSTFSSP_DINT_idx], "long_name", "Fast-FSSP Integral Diameter";
    ncdf_attput, ncdf, vars[FSTFSSP_DINT_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_DINT_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_DINT_idx], "CellSizeRange", fastfssprange;

    vars[FSTFSSP_DAVG_idx] = ncdf_vardef(ncdf, "FSTFSSP_DAVG", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_DAVG_idx], "units", "um";
    ncdf_attput, ncdf, vars[FSTFSSP_DAVG_idx], "long_name", "Fast-FSSP Mean Particle Diameter";
    ncdf_attput, ncdf, vars[FSTFSSP_DAVG_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_DAVG_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_DAVG_idx], "CellSizeRange", fastfssprange;

    vars[FSTFSSP_REFF_idx] = ncdf_vardef(ncdf, "FSTFSSP_REFF", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_REFF_idx], "units", "um";
    ncdf_attput, ncdf, vars[FSTFSSP_REFF_idx], "long_name", "Fast-FSSP Effective Radius";
    ncdf_attput, ncdf, vars[FSTFSSP_REFF_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_REFF_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_REFF_idx], "CellSizeRange", fastfssprange;

    vars[FSTFSSP_DVOL_idx] = ncdf_vardef(ncdf, "FSTFSSP_DVOL", [dims[dim_Time_idx], dims[dim_sps10_idx]], /float);
    ncdf_attput, ncdf, vars[FSTFSSP_DVOL_idx], "units", "um";
    ncdf_attput, ncdf, vars[FSTFSSP_DVOL_idx], "long_name", "Fast-FSSP Mean Volume Diameter";
    ncdf_attput, ncdf, vars[FSTFSSP_DVOL_idx], "Category", "PMS Probe";
    ncdf_attput, ncdf, vars[FSTFSSP_DVOL_idx], "CellSizeUnits", "micrometers";
    ncdf_attput, ncdf, vars[FSTFSSP_DVOL_idx], "CellSizeRange", fastfssprange;

    ; end of definition mode
    print, flights[flight] + ": opening " + filename + ".nc"
    ncdf_control, ncdf, /endef;
    for idx = 0, n_elements(indices_0d) - 1 do begin
      ncdf_varput, ncdf, vars[indices_0d[idx]], offs[indices_0d[idx]];
    endfor

    ; text file second-pass
    i = 0L;
    point_lun, unit, 0;
    numrec = 0L
    print, flights[flight] + ": 2nd pass on " + filename
    while eof(unit) eq 0 do begin
      ; reading line data
      readf, unit, format='(%"%s")', line;
      ; skipping comments, reading and playing with data, writing to ncdf file 
      if (strmid(line, 0, 1) ne '#') then begin
        reads, line, tmpdata1;
        tmpdata10[*, i] = tmpdata1[*];
        i = i + 1L;
        if (i eq 10L) then begin
          i = 0L;
          ; 1-D variables
          for idx = 0, n_elements(indices_long) - 1 do begin
            ncdf_varput, $
              ncdf, $
              vars[indices_long[idx]], $
              long(offs[indices_long[idx]] + mplr[indices_long[idx]] * (tmpdata10[mapl[indices_long[idx]] : mapr[indices_long[idx]], 0L])), $
              offset=*(ncov[indices_long[idx]]);
            (*(ncov[indices_long[idx]]))[0L] = (*(ncov[indices_long[idx]]))[0L] + 1;
          endfor
          ; floating-point variables
          for idx = 0, n_elements(indices_float) - 1L do begin
            for sps=0L, 9L do begin
              (*(ncov[indices_float[idx]]))[1L] = sps;
              for vecidx=0, mapr[indices_float[idx]]-mapl[indices_float[idx]] do begin
                if (n_elements(*(ncov[indices_float[idx]])) eq 3) then (*(ncov[indices_float[idx]]))[2L] = vecidx;
                ncdf_varput, $
                  ncdf, $
                  vars[indices_float[idx]], $
                  offs[indices_float[idx]] + mplr[indices_float[idx]] * double(tmpdata10[mapl[indices_float[idx]]+vecidx, sps]), $
                  offset=*(ncov[indices_float[idx]]);
              endfor
            endfor
            (*(ncov[indices_float[idx]]))[0L] = (*(ncov[indices_float[idx]]))[0L] + 1L;
          endfor
          ; end of write procedures
        endif
        numrec = numrec + 1L;
      endif
    endwhile

    ; closing the text file
    print, flights[flight] + ": closing " + filename
    free_lun, unit;

    ; closing the file
    print, flights[flight] + ": closing " + filename + ".nc"
    ncdf_close, ncdf;

  endfor

end
